/*
 * $Id: javaio.c,v 1.7 2001/05/14 17:58:08 lindahl Exp $
 * 
 *                This source code is part of
 * 
 *                 G   R   O   M   A   C   S
 * 
 *          GROningen MAchine for Chemical Simulations
 * 
 *                        VERSION 3.0
 * 
 * Copyright (c) 1991-2001
 * BIOSON Research Institute, Dept. of Biophysical Chemistry
 * University of Groningen, The Netherlands
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * If you want to redistribute modifications, please consider that
 * scientific software is very special. Version control is crucial -
 * bugs must be traceable. We will be happy to consider code for
 * inclusion in the official distribution, but derived work must not
 * be called official GROMACS. Details are found in the README & COPYING
 * files - if they are missing, get the official version at www.gromacs.org.
 * 
 * To help us fund GROMACS development, we humbly ask that you cite
 * the papers on the package - you can find them in the top README file.
 * 
 * Do check out http://www.gromacs.org , or mail us at gromacs@gromacs.org .
 * 
 * And Hey:
 * Gromacs Runs On Most of All Computer Systems
 */
static char *SRCID_javaio_c = "$Id: javaio.c,v 1.7 2001/05/14 17:58:08 lindahl Exp $";
#include <stdio.h>
#include "typedefs.h"
#include "smalloc.h"
#include "string.h"
#include "string2.h"
#include "readinp.h"
#include "filenm.h"
#include "copyrite.h"
#include "wman.h"

extern char *check_tty(char *s);

static void doindent(FILE *out,int indent,char *file,int line)
{
  int i;

  if (indent < 0)
    fatal_error(0,"Negative indent in %s, line %d",file,line);
  
  for(i=0; (i<indent); i++) 
    fprintf(out," "); 
  fflush(out);
}

#define INDENT() doindent(out,indent,__FILE__,__LINE__)
#define WLN() fprintf(out,"\n")

static void comment(FILE *out,char *s,int indent)
{
  INDENT();
  fprintf(out,"// %s\n",s);
}

#define COMMENT(s) comment(out,s,indent)

void write_java(FILE *out,
		char *program,
		int nldesc,char *desc[],
		int nfile,t_filenm fnm[],
		int npargs,t_pargs pa[],
		int nbug,char *bugs[])
{
#define NCOL 2
  static char *titlestr[NCOL] = { "Type","Filename" };
  int cwidth[NCOL]            = { 2, 3 };
  int cx[NCOL]                = { 0, 2 };
  static char *boolstr[2] = { "false", "true" };
  
  char *ptr;
  char class[256];
  int  i,j,ibag,indent=0;
  int  nBool,nStr;
  
  /* Count the number of booleans (checkboxes!) */
  nBool = 0;
  for(i=0; (i<npargs); i++)
    if (pa[i].type == etBOOL)
      nBool++;
  nStr = npargs - nBool;
  
  /* class name */
  sprintf(class,"%s",program);
  
  COMMENT("");
  fprintf(out,"// Java code generated by GROMACS program %s\n",program);
  fprintf(out,"// %s\n",bromacs());
  COMMENT("Copyright 1991-1997 University of Groningen");
  COMMENT("");
  WLN();
  
  /* import classes */
  fprintf(out,"import java.awt.*;\n");
  fprintf(out,"import java.lang.*;\n");
  fprintf(out,"import java.util.Vector;\n");
  fprintf(out,"import GCCFileDialog;\n");
  fprintf(out,"import gmx_help;\n");
  fprintf(out,"import gmx_fnm;\n");
  fprintf(out,"import gmx_lab;\n");
  fprintf(out,"import gmx_cb;\n");
  fprintf(out,"import gmx_tf;\n");
  WLN();

  /* declaration of class */
  COMMENT("Declaration of class");
  fprintf(out,"class %s extends Frame {\n",class);
  indent+=2;
  
  /* variables in every dialog */
  COMMENT("These variables go into every dialog");
  INDENT();
  fprintf(out,"public int       nFile=%d,nBugs=%d,nHelp=%d;\n",
	  nfile,nbug,nldesc);
  INDENT(); 
  fprintf(out,"public String    strProg=\"%s\";\n",program);
  INDENT(); 
  fprintf(out,"public String    strHelp[];\n");
  INDENT();
  fprintf(out,"public Button    bnOK,bnCancel,bnHelp;\n");
  if (nbug) {
    INDENT();
    fprintf(out,"public Button    bnBugs;\n");
    INDENT();
    fprintf(out,"public String    strBugs[];\n");
  }
  WLN();
  
  COMMENT("Variables depending on input vars");
  if (nBool) {
    INDENT(); 
    fprintf(out,"private gmx_opt theBools[];\n");
  }
  if (nStr) {
    INDENT(); 
    fprintf(out,"private gmx_opt theOpts[];\n");
  }
  if (nfile) {
    INDENT(); 
    fprintf(out,"private gmx_fnm theFiles[];\n");
  }
  WLN();

  COMMENT("Begin constructor");
  INDENT();
  fprintf(out,"%s()\n",class);
  INDENT();
  fprintf(out,"{\n");
  
  indent+=2;
  COMMENT("Local Declarations");
  INDENT();
  fprintf(out,"int fnmIndex;\n");
  WLN();

  INDENT();
  fprintf(out,"setTitle(\"%s\");\n",program);
  WLN();

  COMMENT("Help strings");
  if (nldesc) {
    INDENT();
    fprintf(out,"strHelp = new String[%d];\n",nldesc);
    for(i=0; (i<nldesc); i++) {
      ptr = strdup(desc[i]);
      INDENT();
      fprintf(out,"strHelp[%d] = \"%s\";\n",i,ptr);
      sfree(ptr);
    }
  }
  else {
    INDENT();
    fprintf(out,"strHelp = new String[1];\n");
    INDENT();
    fprintf(out,"strHelp[0] = \"Sorry, no help for %s\";\n",program);
  }
  WLN();

  if (nbug) {
    COMMENT("Known bugs");
    INDENT();
    fprintf(out,"strBugs = new String[%d];\n",nbug);
    for(i=0; (i<nbug); i++) {
      ptr = strdup(bugs[i]);
      INDENT();
      fprintf(out,"strBugs[%d] = \"%s\";\n",i,ptr);
      sfree(ptr);
    }
  }
  WLN();

  COMMENT("The elements of the graphics frame");
  INDENT();
  fprintf(out,"theFiles  = new gmx_fnm[%d];\n",nfile);
  for(i=0; (i<nfile); i++) {
    INDENT();
    fprintf(out,"theFiles[%d] = "
	    "new gmx_fnm(\"%s\",\"%s\",\"%s\",\"%s\",%s,this);\n",
	    i,fnm[i].opt,fnm[i].fn,ftp2desc(fnm[i].ftp),fileopt(fnm[i].flag),
	    boolstr[(fnm[i].flag & ffOPT) == ffOPT]);
  }
  WLN();

  COMMENT("Make the panels");
  INDENT();
  fprintf(out,"Panel topP = new Panel();\n");
  INDENT();
  fprintf(out,"GridBagLayout gbl = new GridBagLayout();\n");
  INDENT();
  fprintf(out,"GridBagConstraints bag[] = new GridBagConstraints[%d];\n",
	  NCOL*(nfile+1));
  INDENT();
  fprintf(out,"Label lbFile[] = new Label[%d];\n",NCOL);
  WLN();
  
  INDENT();
  fprintf(out,"topP.setLayout(gbl);\n");
  {
    int yyy=0,height=1;
        
    for(ibag=0; (ibag<NCOL); ibag++) {
      INDENT();
      fprintf(out,"bag[%d]            = new GridBagConstraints();\n",ibag);
      INDENT();
      fprintf(out,"bag[%d].gridx      = %d;\n",ibag,cx[ibag]);
      INDENT();
      fprintf(out,"bag[%d].gridy      = %d;\n",ibag,yyy);
      INDENT();
      fprintf(out,"bag[%d].gridwidth  = %d;\n",ibag,cwidth[ibag]);
      INDENT();
      fprintf(out,"bag[%d].gridheight = %d;\n",ibag,height);
      INDENT();
      fprintf(out,"bag[%d].fill       = 1;\n",ibag);
      INDENT();
      fprintf(out,"lbFile[%d] = new Label(\"%s\");\n",ibag,titlestr[ibag]);
      INDENT();
      fprintf(out,"topP.add(lbFile[%d]);\n",ibag);
      INDENT();
      fprintf(out,"gbl.setConstraints(lbFile[%d],bag[%d]);\n",ibag,ibag);
    }
    yyy ++;
    WLN();
    
    INDENT();
    fprintf(out,"fnmIndex = 0;\n");
    for(i=0; (i<nfile); i++) {
      INDENT();
      fprintf(out,"bag[%d] = (GridBagConstraints) bag[%d].clone();\n",
	      ibag,ibag % NCOL);
      INDENT();
      fprintf(out,"bag[%d].gridy = %d;\n",ibag,yyy);
      INDENT();
      fprintf(out,"topP.add(theFiles[%d].fnmButton());\n",i);
      INDENT();
      fprintf(out,"gbl.setConstraints(theFiles[%d].fnmButton(),bag[%d]);\n",
	      i,ibag);
      ibag++;
      
      INDENT();
      fprintf(out,"bag[%d] = (GridBagConstraints) bag[%d].clone();\n",
	      ibag,ibag % NCOL);
      INDENT();
      fprintf(out,"bag[%d].gridy = %d;\n",ibag,yyy);
      INDENT();
      fprintf(out,"topP.add(theFiles[%d].fnmTextField());\n",i);
      INDENT();
      fprintf(out,"gbl.setConstraints(theFiles[%d].fnmTextField(),bag[%d]);\n",
	      i,ibag);
      ibag++;
      
      yyy++;
    }
    INDENT();
    fprintf(out,"add(\"North\",topP);\n");
    WLN();
  
    if (nBool) {
      COMMENT("Panel time once more");
      INDENT();
      fprintf(out,"Panel   boolP = new Panel();\n");
      INDENT();
      fprintf(out,"boolP.setLayout(new GridLayout(0,%d));\n",7);
      INDENT();
      fprintf(out,"theBools = new gmx_opt[%d];\n",nBool);
      WLN();
      
      for(i=j=0; (i<npargs); i++) {
	if (pa[i].type == etBOOL) {
	  INDENT();
	  fprintf(out,"theBools[%d] = new gmx_opt(\"%s\",\"%s\",this,%s);\n",
		  j,pa[i].option,pa[i].desc,boolstr[*(pa[i].u.b)]);
	  INDENT();
	  fprintf(out,"boolP.add(theBools[%d].gmxCB());\n",j);
	  j++;
	}
      }
      INDENT();
      fprintf(out,"add(\"Center\",boolP);\n");
      WLN();
    }
    
    COMMENT("Panel time once more");
    INDENT();
    fprintf(out,"Panel   buttonP = new Panel();\n");
    WLN();
    
    COMMENT("Buttons for every application");
    INDENT();
    fprintf(out,"bnOK     = new Button(\"OK\");\n");
    INDENT();
    fprintf(out,"buttonP.add(bnOK);\n");

    INDENT();
    fprintf(out,"bnHelp   = new Button(\"Help\");\n");
    INDENT();
    fprintf(out,"buttonP.add(bnHelp);\n");
    
    if (nbug) {
      INDENT();
      fprintf(out,"bnBugs   = new Button(\"Bugs\");\n");
      INDENT();
      fprintf(out,"buttonP.add(bnBugs);\n");
    }
    
    INDENT();
    fprintf(out,"bnCancel = new Button(\"Cancel\");\n");
    INDENT();
    fprintf(out,"buttonP.add(bnCancel);\n");
    INDENT();
    fprintf(out,"add(\"South\",buttonP);\n");
    WLN();
  }
  
  COMMENT("Initiate size and location, and then GO!");
  INDENT();
  fprintf(out,"setSize(new Dimension(500,200));\n");
  INDENT();
  fprintf(out,"setLocation(new Point(200,200));\n");
  INDENT();
  fprintf(out,"pack();\n");
  INDENT();
  fprintf(out,"show();\n");

  /* End constructor */
  COMMENT("End constructor");

  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  WLN();
  
  /*
   *   E V E N T      H A N D L E R 
   */
  COMMENT("The event handler");
  INDENT();
  fprintf(out,"public boolean handleEvent(Event evt)\n");
  INDENT();
  fprintf(out,"{\n");
  indent += 2;
  INDENT();
  fprintf(out,"GCCFileDialog fileDialog;\n");
  INDENT();
  fprintf(out,"String strFile;\n");
  INDENT();
  fprintf(out,"int    i;\n");
  WLN();

  INDENT();
  fprintf(out,"strFile  = new String();\n");
  
  INDENT();
  fprintf(out,"switch (evt.id) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"case Event.ACTION_EVENT:\n");
  INDENT();
  fprintf(out,"for(i=0; (i<nFile); i++) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"if (evt.target == theFiles[i].fnmButton()) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"fileDialog = new GCCFileDialog(this,\"Open file...\");\n");
  INDENT();
  fprintf(out,"theFiles[i].setFilename(fileDialog.getFilename());\n");
  INDENT();
  fprintf(out,"System.out.println(\"File: \" + theFiles[i].getFilename());\n");
  INDENT();
  fprintf(out,"return true;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");

  INDENT();
  fprintf(out,"if (\"OK\".equals(evt.arg)) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"for(i=0; (i<nFile); i++) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"String tmp = theFiles[i].getFilename();\n");
  INDENT();
  fprintf(out,"System.out.println(\"File \" + i + tmp);\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");

  INDENT();
  fprintf(out,"Runtime r = Runtime.getRuntime();\n");
  INDENT();
  fprintf(out,"Process p = null;\n");

  INDENT();
  fprintf(out,"String cmd = strProg;\n");
  INDENT();
  fprintf(out,"for(i=0; (i<nFile); i++) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"cmd = cmd+\" \"+theFiles[i].getCmd();\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");

  INDENT();
  fprintf(out,"try {\n");
  indent+=2;
  INDENT();
  fprintf(out,"System.out.println(\"Going to exec \"+cmd);\n");
  INDENT();
  fprintf(out,"p = r.exec(cmd);\n");
  indent-=2;
  INDENT();
  fprintf(out,"} catch (Exception e) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"System.out.println(\"error executing \" + cmd);\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  INDENT();
  fprintf(out,"dispose();\n");
  INDENT();
  fprintf(out,"return true;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  WLN();
  
  INDENT();
  fprintf(out,"else if (\"Cancel\".equals(evt.arg)) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"dispose();\n");
  INDENT();
  fprintf(out,"return true;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  WLN();
  
  INDENT();
  fprintf(out,"else if (\"Help\".equals(evt.arg)) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"new gmx_help(\"Help for\"+strProg,nHelp,strHelp);\n");
  INDENT();
  fprintf(out,"return true;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  WLN();
  
  INDENT();
  fprintf(out,"else if (\"Bugs\".equals(evt.arg)) {\n");
  indent+=2;
  INDENT();
  fprintf(out,"new gmx_help(\"Known bugs in \"+strProg,nBugs,strBugs);\n");
  INDENT();
  fprintf(out,"return true;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  WLN();
  
  
  INDENT();
  fprintf(out,"else return false;\n");
  WLN();

  INDENT();
  fprintf(out,"case Event.WINDOW_DESTROY:\n");
  INDENT();
  fprintf(out,"dispose();\n");
  INDENT();
  fprintf(out,"break;\n");
  INDENT();
  fprintf(out,"default: return false;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");
  INDENT();
  fprintf(out,"return false;\n");
  indent-=2;
  INDENT();
  fprintf(out,"}\n");

  /* End class */
  INDENT();
  fprintf(out,"// End class %s\n",class);

  indent-=2;
  INDENT();
  fprintf(out,"}\n");

  
}

#undef INDENT
